@@ -113,7 +113,7 @@ sub new
                  postmaster => 'postmaster@example.com',
                  quota_db => 'quotalegacy',
                  quota_use_conversations => 'yes',
-                 quotawarn => '98',
+                 quotawarnpercent => '98',
                  reverseacls => 'yes',
                  rfc3028_strict => 'no',
                  savedate => 'yes',
@@ -121,7 +121,7 @@ sub new
                  sieve_utf8fileinto => 'yes',
                  sieve_use_lmtp_reject => 'no',
                  sievenotifier => 'mailto',
-                 sieve_maxscriptsize => '1024',
+                 sieve_maxscriptsize => '1024K',
                  sieve_vacation_min_response => '60',
                  specialusealways => 'yes',
                  specialuse_extra => '\\XChats \\XTemplates \\XNotes',
@@ -180,6 +180,7 @@ my @default_using = qw(
     urn:ietf:params:jmap:mail
     urn:ietf:params:jmap:submission
     https://cyrusimap.org/ns/jmap/blob
+    urn:ietf:params:jmap:calendars
     https://cyrusimap.org/ns/jmap/contacts
     https://cyrusimap.org/ns/jmap/calendars
     https://cyrusimap.org/ns/jmap/mail
@@ -380,7 +381,9 @@ sub test_issue_LP52545479
         ['CalendarEvent/set', {
             create => {
                 event1 => {
-                    calendarId => $calendarId,
+                    calendarIds => {
+                        $calendarId => JSON::true,
+                    },
                     uid => '58ADE31-custom-UID',
                     title => 'event1',
                     start => '2015-11-07T09:00:00',
@@ -1328,6 +1331,7 @@ sub test_rename_deepfolder_intermediates_rightnow
 
 sub test_imap_list_notes
     :min_version_3_1 :needs_component_sieve
+    :needs_component_httpd
 {
     my ($self) = @_;
 
@@ -1410,6 +1414,7 @@ sub test_imap_list_notes
 
 sub test_cyr_expire_delete_findpaths_legacy
     :DelayedDelete :min_version_3_5 :MailboxLegacyDirs
+    :needs_component_httpd
 {
     my ($self) = @_;
 
@@ -1479,6 +1484,7 @@ sub test_cyr_expire_delete_findpaths_legacy
 
 sub test_cyr_expire_delete_findpaths_nolegacy
     :DelayedDelete :min_version_3_5 :NoMailboxLegacyDirs
+    :needs_component_httpd
 {
     my ($self) = @_;
 
@@ -1552,6 +1558,7 @@ sub test_cyr_expire_delete_findpaths_nolegacy
 
 sub test_sync_reset_legacy
     :DelayedDelete :min_version_3_5 :MailboxLegacyDirs
+    :needs_component_replication
 {
     my ($self) = @_;
 
@@ -1593,6 +1600,7 @@ sub test_sync_reset_legacy
 
 sub test_sync_reset_nolegacy
     :DelayedDelete :min_version_3_5 :NoMailboxLegacyDirs
+    :needs_component_replication
 {
     my ($self) = @_;
 
@@ -1641,6 +1649,7 @@ sub test_sync_reset_nolegacy
 
 sub test_relocate_legacy_nodomain
     :DelayedDelete :min_version_3_5 :MailboxLegacyDirs
+    :needs_component_httpd
 {
     my ($self) = @_;
 
@@ -1682,6 +1691,7 @@ sub test_relocate_legacy_nodomain
 
 sub test_relocate_legacy_domain
     :DelayedDelete :min_version_3_5 :MailboxLegacyDirs
+    :needs_component_httpd
 {
     my ($self) = @_;
 
@@ -1723,6 +1733,7 @@ sub test_relocate_legacy_domain
 
 sub test_relocate_legacy_nosearchdb
     :DelayedDelete :min_version_3_5 :MailboxLegacyDirs
+    :needs_component_httpd
 {
     my ($self) = @_;
 
@@ -1767,6 +1778,7 @@ sub test_relocate_legacy_nosearchdb
 
 sub test_relocate_messages_still_exist
     :DelayedDelete :min_version_3_5 :MailboxLegacyDirs
+    :needs_component_httpd
 {
     my ($self) = @_;
 
@@ -1881,7 +1893,7 @@ sub test_rename_quotaroot
 }
 
 sub test_search_deleted_folder
-    :DelayedDelete :min_version_3_5 :NoMailboxLegacyDirs
+    :DelayedDelete :min_version_3_5 :NoMailboxLegacyDirs :needs_component_jmap
 {
     my ($self) = @_;
 
