@@ -102,6 +102,7 @@ sub test_capabilities
         }, "R1"]
     ], [
         'urn:ietf:params:jmap:core',
+        'urn:ietf:params:jmap:calendars',
         'https://cyrusimap.org/ns/jmap/calendars',
     ]);
     $self->assert_str_equals('error', $res->[0][0]);
@@ -187,8 +188,10 @@ sub test_get_session
     $self->assert(exists $coreCapability->{collationAlgorithms});
     $self->assert_deep_equals({}, $capabilities->{'urn:ietf:params:jmap:mail'});
     $self->assert_deep_equals({}, $capabilities->{'urn:ietf:params:jmap:submission'});
+    $self->assert_deep_equals({}, $capabilities->{'urn:ietf:params:jmap:calendars'});
     $self->assert_deep_equals({}, $capabilities->{'https://cyrusimap.org/ns/jmap/contacts'});
-    $self->assert_deep_equals({}, $capabilities->{'https://cyrusimap.org/ns/jmap/calendars'});
+    $self->assert_deep_equals({ isRFC => JSON::true },
+        , $capabilities->{'https://cyrusimap.org/ns/jmap/calendars'});
     if ($buildinfo->get('component', 'sieve')) {
         $self->assert_deep_equals({}, $capabilities->{'urn:ietf:params:jmap:vacationresponse'});
         if ($maj > 3 || ($maj == 3 && $min >= 3)) {
@@ -201,6 +204,8 @@ sub test_get_session
     my $expect_primaryAccounts = {
         'urn:ietf:params:jmap:mail' => 'cassandane',
         'urn:ietf:params:jmap:submission' => 'cassandane',
+        'urn:ietf:params:jmap:calendars' => 'cassandane',
+        'urn:ietf:params:jmap:principals' => 'cassandane',
         'https://cyrusimap.org/ns/jmap/contacts' => 'cassandane',
         'https://cyrusimap.org/ns/jmap/calendars' => 'cassandane',
     };
@@ -241,6 +246,7 @@ sub test_get_session
     if ($buildinfo->get('component', 'sieve')) {
         $self->assert_not_null($accountCapabilities->{'urn:ietf:params:jmap:vacationresponse'});
     }
+    $self->assert_not_null($accountCapabilities->{'urn:ietf:params:jmap:calendars'});
     $self->assert_not_null($accountCapabilities->{'https://cyrusimap.org/ns/jmap/contacts'});
     $self->assert_not_null($accountCapabilities->{'https://cyrusimap.org/ns/jmap/calendars'});
 
@@ -256,6 +262,7 @@ sub test_get_session
         $self->assert_null($accountCapabilities->{'urn:ietf:params:jmap:vacationresponse'});
     }
     $self->assert_null($accountCapabilities->{'https://cyrusimap.org/ns/jmap/contacts'});
+    $self->assert_not_null($accountCapabilities->{'urn:ietf:params:jmap:calendars'});
     $self->assert_not_null($accountCapabilities->{'https://cyrusimap.org/ns/jmap/calendars'});
 
     # Account 2: read/write mail
@@ -269,6 +276,7 @@ sub test_get_session
     if ($buildinfo->get('component', 'sieve')) {
         $self->assert_null($accountCapabilities->{'urn:ietf:params:jmap:vacationresponse'});
     }
+    $self->assert_null($accountCapabilities->{'urn:ietf:params:jmap:calendars'});
     $self->assert_null($accountCapabilities->{'https://cyrusimap.org/ns/jmap/contacts'});
     $self->assert_null($accountCapabilities->{'https://cyrusimap.org/ns/jmap/calendars'});
 }
@@ -470,12 +478,24 @@ sub test_identity_get
         'urn:ietf:params:jmap:submission',
     ];
 
-    my $res = $jmap->CallMethods([['Identity/get', {}, "R1"]], $using);
+    my $res = $jmap->CallMethods([
+        ['Identity/get', { }, 'R1'],
+        ['Identity/get', { ids => undef }, 'R2'],
+        ['Identity/get', { ids => [] }, 'R3'],
+    ], $using);
 
     $self->assert_str_equals('Identity/get', $res->[0][0]);
     $self->assert_num_equals(1, scalar @{$res->[0][1]{list}});
-    $self->assert_str_equals('cassandane', $res->[0][1]{list}[0]{id}); $self->assert_not_null($res->[0][1]->{state});
+    $self->assert_str_equals('cassandane', $res->[0][1]{list}[0]{id});
+    $self->assert_not_null($res->[0][1]->{state});
     $self->assert_str_equals('R1', $res->[0][2]);
+
+    $self->assert_num_equals(1, scalar @{$res->[1][1]{list}});
+    $self->assert_str_equals('cassandane', $res->[1][1]{list}[0]{id});
+    $self->assert_not_null($res->[1][1]->{state});
+
+    $self->assert_deep_equals([], $res->[2][1]{list});
+    $self->assert_not_null($res->[2][1]->{state});
 }
 
 sub test_sessionstate
@@ -674,15 +694,15 @@ sub test_blob_set_basic
 
     xlog "Test without capability";
     my $jmap = $self->{jmap};
-    my $res = $jmap->CallMethods([['Blob/set', { create => { b1 => { 'data:asText' => 'hello world' } } }, 'R1']]);
+    my $res = $jmap->CallMethods([['Blob/upload', { create => { b1 => { data => [{'data:asText' => 'hello world'}] } } }, 'R1']]);
     $self->assert_str_equals($res->[0][0], 'error');
 
     # XXX: this will be replaced with the upstream one
     $jmap->AddUsing('https://cyrusimap.org/ns/jmap/blob');
 
-    xlog "Regular Blob/set works and returns a blobId";
-    $res = $jmap->CallMethods([['Blob/set', { create => { b1 => { 'data:asText' => 'hello world' } } }, 'R1']]);
-    $self->assert_str_equals('Blob/set', $res->[0][0]);
+    xlog "Regular Blob/upload works and returns a blobId";
+    $res = $jmap->CallMethods([['Blob/upload', { create => { b1 => { data => [{'data:asText' => 'hello world'}] } } }, 'R1']]);
+    $self->assert_str_equals('Blob/upload', $res->[0][0]);
     $self->assert_not_null($res->[0][1]{created}{b1}{id});
 }
 
@@ -696,7 +716,7 @@ sub test_blob_lookup
     my $talk = $store->get_client();
     my $inbox = 'INBOX';
 
-    xlog $self, "Generate a email in $inbox via IMAP";
+    xlog $self, "Generate an email in $inbox via IMAP";
     my %exp_sub;
     $store->set_folder($inbox);
     $store->_select();
@@ -757,7 +777,7 @@ sub test_blob_get
     my $talk = $store->get_client();
     my $inbox = 'INBOX';
 
-    xlog $self, "Generate a email in $inbox via IMAP";
+    xlog $self, "Generate an email in $inbox via IMAP";
     my %exp_sub;
     $store->set_folder($inbox);
     $store->_select();
@@ -802,6 +822,17 @@ sub test_blob_set_complex
     my $self = shift;
     my $jmap = $self->{jmap};
 
+    # GET supported digest algorithms from session object
+    my $RawRequest = {
+        headers => {
+            'Authorization' => $jmap->auth_header(),
+        },
+        content => '',
+    };
+    my $RawResponse = $jmap->ua->get($jmap->uri(), $RawRequest);
+    my $session = eval { decode_json($RawResponse->{content}) };
+    my %algs = map { $_ => 1 } @{$session->{capabilities}->{'https://cyrusimap.org/ns/jmap/blob'}->{supportedDigestAlgorithms}};
+
     # XXX: this will be replaced with the upstream one
     $jmap->AddUsing('https://cyrusimap.org/ns/jmap/blob');
 
@@ -810,44 +841,160 @@ sub test_blob_set_complex
 
     my $res;
 
-    xlog "Regular Blob/set works and returns the right data";
+    xlog "Regular Blob/upload works and returns the right data";
     $res = $jmap->CallMethods([
-      ['Blob/set', { create => { b1 => { 'data:asText' => $data } } }, 'S1'],
+      ['Blob/upload', { create => { b1 => { data => [{'data:asText' => $data}] } } }, 'S1'],
       ['Blob/get', { ids => ['#b1'], properties => [ 'data:asText', 'size' ] }, 'G1'],
     ]);
-    $self->assert_str_equals('Blob/set', $res->[0][0]);
+    $self->assert_str_equals('Blob/upload', $res->[0][0]);
     $self->assert_str_equals('Blob/get', $res->[1][0]);
     $self->assert_str_equals($data, $res->[1][1]{list}[0]{'data:asText'});
     $self->assert_num_equals(length $data, $res->[1][1]{list}[0]{size});
 
-    xlog "Base64 Blob/set works and returns the right data";
+    xlog "Base64 Blob/upload works and returns the right data";
+    my $props = [ 'data:asText', 'size' ];
+    if ($algs{'md5'}) {
+        push @{$props}, 'digest:md5';
+    }
+    if ($algs{'sha'}) {
+        push @{$props}, 'digest:sha';
+    }
+    if ($algs{'sha-256'}) {
+        push @{$props}, 'digest:sha-256';
+    }
+
     $res = $jmap->CallMethods([
-      ['Blob/set', { create => { b2 => { 'data:asBase64' => $bdata } } }, 'S2'],
-      ['Blob/get', { ids => ['#b2'], properties => [ 'data:asText', 'size' ] }, 'G2'],
+      ['Blob/upload', { create => { b2 => { data => [{'data:asBase64' => $bdata}] } } }, 'S2'],
+      ['Blob/get', { ids => ['#b2'], properties => $props }, 'G2'],
+      ['Blob/get', { ids => ['#b2'], offset => 4, length => 9, properties => $props }, 'G2'],
     ]);
-    $self->assert_str_equals('Blob/set', $res->[0][0]);
+    $self->assert_str_equals('Blob/upload', $res->[0][0]);
     $self->assert_str_equals('Blob/get', $res->[1][0]);
     $self->assert_str_equals($data, $res->[1][1]{list}[0]{'data:asText'});
     $self->assert_num_equals(length $data, $res->[1][1]{list}[0]{size});
+    $self->assert_str_equals("quick bro", $res->[2][1]{list}[0]{'data:asText'});
+    if ($algs{'md5'}) {
+        $self->assert_str_equals("tTNHgg3iNoIdHFn81iQD9A==", $res->[2][1]{list}[0]{'digest:md5'});
+    }
+    if ($algs{'sha'}) {
+        $self->assert_str_equals("QiRAPtfyX8K6tm1iOAtZ87Xj3Ww=", $res->[2][1]{list}[0]{'digest:sha'});
+    }
+    if ($algs{'sha-256'}) {
+        $self->assert_str_equals("gdg9INW7lwHK6OQ9u0dwDz2ZY/gubi0En0xlFpKt0OA=", $res->[2][1]{list}[0]{'digest:sha-256'});
+    }
 
-    xlog "Complex catenate expression works and returns the right data";
+    xlog "Complex expression works and returns the right data";
     my $target = "How quick was that?";
     $res = $jmap->CallMethods([
-      ['Blob/set', { create => { b4 => { 'data:asText' => $data } } }, 'S4'],
-      ['Blob/set', { create => { cat => { catenate => [
+      ['Blob/upload', { create => { b4 => { data => [{'data:asText' => $data}] } } }, 'S4'],
+      ['Blob/upload', { create => { mult => { data => [
         { 'data:asText' => 'How' },                      # 'How'
         { 'blobId' => '#b4', offset => 3, length => 7 }, # ' quick '
         { 'data:asText' => "was t" },                    # 'was t'
         { 'blobId' => '#b4', offset => 1, length => 1 }, # 'h'
         { 'data:asBase64' => encode_base64('at?', '') }, # 'at?'
       ] } } }, 'CAT'],
-      ['Blob/get', { ids => ['#cat'], properties => [ 'data:asText', 'size' ] }, 'G4'],
+      ['Blob/get', { ids => ['#mult'], properties => [ 'data:asText', 'size' ] }, 'G4'],
     ]);
-    $self->assert_str_equals('Blob/set', $res->[0][0]);
-    $self->assert_str_equals('Blob/set', $res->[1][0]);
+    $self->assert_str_equals('Blob/upload', $res->[0][0]);
+    $self->assert_str_equals('Blob/upload', $res->[1][0]);
     $self->assert_str_equals('Blob/get', $res->[2][0]);
     $self->assert_str_equals($target, $res->[2][1]{list}[0]{'data:asText'});
     $self->assert_num_equals(length $target, $res->[2][1]{list}[0]{size});
 }
 
+sub test_blob_upload_repair_acl
+    :min_version_3_7 :needs_component_jmap :JMAPExtensions
+{
+    my $self = shift;
+    my $jmap = $self->{jmap};
+    my $admin = $self->{adminstore}->get_client();
+
+    $jmap->Upload("hello", "application/data");
+
+    my $file = abs_path('data/mime/repair_acl.eml');
+    open(my $fh, '<', $file);
+    local $/;
+    my $binary = <$fh>;
+    close($fh);
+
+    xlog "Assert that uploading duplicates does not fail";
+    $admin->setacl("user.cassandane.#jmap", "cassandane", "lrswkcni") or die;
+    my $res = $jmap->Upload($binary);
+    my $blobId = $res->{blobId};
+    $res = $jmap->Upload($binary, "message/rfc822");
+    $self->assert_str_equals($blobId, $res->{blobId});
+
+    xlog "Assert ACLs got repaired";
+    my %acl = @{$admin->getacl("user.cassandane.#jmap")};
+    $self->assert_str_equals("lrswitedn", $acl{cassandane});
+}
+
+sub test_blob_upload_type
+    :min_version_3_7 :needs_component_jmap :JMAPExtensions
+{
+    my $self = shift;
+    my $jmap = $self->{jmap};
+
+    xlog "Assert client-supplied type is returned";
+    my $res = $jmap->Upload("blob1", "text/plain");
+    $self->assert_str_equals("text/plain", $res->{type});
+
+    xlog "Assert client-supplied type is normalized";
+    $res = $jmap->Upload("blob1", "text/plain;charset=latin1");
+    $self->assert_str_equals("text/plain", $res->{type});
+
+    xlog "Assert default server type";
+    my $httpReq = {
+        headers => {
+            'Authorization' => $jmap->auth_header(),
+        },
+        content => 'blob2',
+    };
+    my $httpRes = $jmap->ua->post($jmap->uploaduri('cassandane'), $httpReq);
+    if ($ENV{DEBUGJMAP}) {
+        warn "JMAP " . Dumper($httpReq, $httpRes);
+    }
+    $res = eval { decode_json($httpRes->{content}) };
+    $self->assert_str_equals("application/octet-stream", $res->{type});
+}
+
+sub test_get_session_rename_race
+    :AllowMoves :min_version_3_7 :needs_component_jmap
+{
+    # Test fetching the JMAP session during/after a user has been renamed
+    # but before the authentication credentials have been changed.
+    # In this case, we should return 503 rather than 500.
+    my ($self) = @_;
+
+    my $jmap = $self->{jmap};
+    my $admintalk = $self->{adminstore}->get_client();
+
+    # GET session
+    my $RawRequest = {
+        headers => {
+            'Authorization' => $jmap->auth_header(),
+        },
+        content => '',
+    };
+    my $RawResponse = $jmap->ua->get($jmap->uri(), $RawRequest);
+    if ($ENV{DEBUGJMAP}) {
+        warn "JMAP " . Dumper($RawRequest, $RawResponse);
+    }
+    $self->assert_str_equals('200', $RawResponse->{status});
+    my $session = eval { decode_json($RawResponse->{content}) };
+    $self->assert_not_null($session);
+
+    my $res = $admintalk->rename('user.cassandane', 'user.newuser');
+    $self->assert(not $admintalk->get_last_error());
+
+    # Try to GET session
+    $RawResponse = $jmap->ua->get($jmap->uri(), $RawRequest);
+    if ($ENV{DEBUGJMAP}) {
+        warn "JMAP " . Dumper($RawRequest, $RawResponse);
+    }
+    $self->assert_str_equals('503', $RawResponse->{status});
+    $self->assert_not_null($RawResponse->{headers}{'retry-after'});
+}
+
 1;
