@@ -65,6 +65,8 @@ my %suppressed;
 # dot-separated.  See skip_before() below for implementation details.
 my %notbefore = (
     't:Email:get:header-header-field-name' => '3.5',
+    't:Email:import:good-imports' => '3.8',
+    't:Email:import:one-fails-another-succeeds' => '3.8',
 );
 
 sub cyrus_version_supports_jmap
