@@ -71,6 +71,7 @@ sub tear_down
 # Test replication of messages APPENDed to the master
 #
 sub test_append
+    :needs_component_replication
 {
     my ($self) = @_;
 
@@ -102,7 +103,7 @@ sub test_append
 # Test handling of replication when append fails due to disk error
 #
 sub test_appendone_diskfull
-    :NoStartInstances :min_version_3_5
+    :NoStartInstances :min_version_3_5 :needs_component_replication
 {
     my ($self) = @_;
 
@@ -177,6 +178,7 @@ EOF
 #
 sub test_appendmulti_diskfull
     :CSyncReplication :NoStartInstances :min_version_3_5
+    :needs_component_replication
 {
     my ($self) = @_;
 
@@ -261,7 +263,7 @@ EOF
 # Test handling of replication when append fails due to disk error
 #
 sub test_syncall_failinguser
-    :NoStartInstances :min_version_3_6
+    :NoStartInstances :min_version_3_6 :needs_component_replication
 {
     my ($self) = @_;
 
@@ -376,6 +378,7 @@ EOF
 # Test replication of messages APPENDed to the master
 #
 sub test_splitbrain
+    :needs_component_replication
 {
     my ($self) = @_;
 
@@ -448,6 +451,7 @@ sub test_splitbrain
 #
 sub test_splitbrain_mailbox
     :min_version_3_1 :max_version_3_4 :NoAltNameSpace
+    :needs_component_replication
 {
     my ($self) = @_;
 
@@ -545,6 +549,7 @@ sub test_splitbrain_mailbox
 # Test replication of messages APPENDed to the master
 #
 sub test_splitbrain_masterexpunge
+    :needs_component_replication
 {
     my ($self) = @_;
 
@@ -621,6 +626,7 @@ sub test_splitbrain_masterexpunge
 # Test replication of messages APPENDed to the master
 #
 sub test_splitbrain_replicaexpunge
+    :needs_component_replication
 {
     my ($self) = @_;
 
@@ -693,6 +699,7 @@ sub test_splitbrain_replicaexpunge
 # Test replication of messages APPENDed to the master
 #
 sub test_splitbrain_bothexpunge
+    :needs_component_replication
 {
     my ($self) = @_;
 
@@ -758,7 +765,7 @@ sub test_splitbrain_bothexpunge
 
 # trying to reproduce error reported in https://git.cyrus.foundation/T228
 sub test_alternate_globalannots
-    :NoStartInstances
+    :NoStartInstances :needs_component_replication
 {
     my ($self) = @_;
 
@@ -779,7 +786,7 @@ sub test_alternate_globalannots
 }
 
 sub test_sieve_replication
-    :needs_component_sieve
+    :needs_component_sieve :needs_component_replication
 {
     my ($self) = @_;
 
@@ -823,7 +830,7 @@ EOF
 }
 
 sub test_sieve_replication_exists
-    :needs_component_sieve
+    :needs_component_sieve :needs_component_replication
 {
     my ($self) = @_;
 
@@ -868,7 +875,7 @@ EOF
 }
 
 sub test_sieve_replication_different
-    :needs_component_sieve
+    :needs_component_sieve :needs_component_replication
 {
     my ($self) = @_;
 
@@ -930,7 +937,7 @@ EOF
 }
 
 sub test_sieve_replication_stale
-    :needs_component_sieve
+    :needs_component_sieve :needs_component_replication
 {
     my ($self) = @_;
 
@@ -996,7 +1003,7 @@ EOF
 }
 
 sub test_sieve_replication_delete_unactivate
-    :needs_component_sieve
+    :needs_component_sieve :needs_component_replication
 {
     my ($self) = @_;
 
@@ -1041,6 +1048,7 @@ EOF
 
 sub test_sieve_replication_unixhs
     :needs_component_sieve :UnixHierarchySep
+    :needs_component_replication
 {
     my ($self) = @_;
 
@@ -1088,6 +1096,7 @@ EOF
 
 sub test_sieve_replication_exists_unixhs
     :needs_component_sieve :UnixHierarchySep
+    :needs_component_replication
 {
     my ($self) = @_;
 
@@ -1138,6 +1147,7 @@ EOF
 
 sub test_sieve_replication_different_unixhs
     :needs_component_sieve :UnixHierarchySep
+    :needs_component_replication
 {
     my ($self) = @_;
 
@@ -1205,6 +1215,7 @@ EOF
 
 sub test_sieve_replication_stale_unixhs
     :needs_component_sieve :UnixHierarchySep
+    :needs_component_replication
 {
     my ($self) = @_;
 
@@ -1276,6 +1287,7 @@ EOF
 
 sub test_sieve_replication_delete_unactivate_unixhs
     :needs_component_sieve :UnixHierarchySep
+    :needs_component_replication
 {
     my ($self) = @_;
 
@@ -1336,7 +1348,7 @@ sub slurp_file
 
 # this test is too tricky to get working on uuid mailboxes
 sub test_replication_mailbox_too_old
-    :max_version_3_4
+    :max_version_3_4 :needs_component_replication
 {
     my ($self) = @_;
 
@@ -1418,7 +1430,7 @@ sub test_replication_mailbox_too_old
 
 # this test is too tricky to get working on uuid mailboxes
 sub test_replication_mailbox_new_enough
-    :max_version_3_4
+    :max_version_3_4 :needs_component_replication
 {
     my ($self) = @_;
 
@@ -1441,6 +1453,7 @@ sub test_replication_mailbox_new_enough
 #* run sync_client on the master again and make sure it successfully syncs up
 
 sub test_replication_repair_zero_msgs
+    :needs_component_replication
 {
     my ($self) = @_;
 
@@ -1462,6 +1475,7 @@ sub test_replication_repair_zero_msgs
 }
 
 sub test_replication_with_modified_seen_flag
+    :needs_component_replication
 {
     my ($self) = @_;
 
@@ -1566,6 +1580,7 @@ sub assert_user_sub_not_exists
 }
 
 sub test_subscriptions
+    :needs_component_replication
 {
     my ($self) = @_;
 
@@ -1662,7 +1677,7 @@ sub test_subscriptions
 }
 
 sub test_subscriptions_unixhs
-    :UnixHierarchySep
+    :UnixHierarchySep :needs_component_replication
 {
     my ($self) = @_;
 
@@ -1762,7 +1777,7 @@ sub test_subscriptions_unixhs
 # wasn't correctly looking only for children of that name, so it would try
 # to delete the wrong user's mailbox.
 sub test_userprefix
-    :DelayedDelete
+    :DelayedDelete :needs_component_replication
 {
     my ($self) = @_;
     $self->{instance}->create_user("ua");
@@ -1804,7 +1819,7 @@ sub test_userprefix
 # wasn't correctly looking only for children of that name, so it would try
 # to delete the wrong user's mailbox.
 sub test_reset_on_master
-    :DelayedDelete :min_version_3_3
+    :DelayedDelete :min_version_3_3 :needs_component_replication
 {
     my ($self) = @_;
     $self->{instance}->create_user("user2");
@@ -1854,7 +1869,7 @@ sub test_reset_on_master
 
 # this is testing a bug where sync_client would abort on zero-length file
 sub test_sync_empty_file
-    :DelayedDelete :min_version_3_3
+    :DelayedDelete :min_version_3_3 :needs_component_replication
 {
     my ($self) = @_;
 
@@ -1869,7 +1884,7 @@ sub test_sync_empty_file
 }
 
 sub test_sync_log_mailbox_with_spaces
-    :DelayedDelete :NoStartInstances
+    :DelayedDelete :NoStartInstances :needs_component_replication
 {
     my ($self) = @_;
 
@@ -1936,6 +1951,7 @@ sub test_sync_log_mailbox_with_spaces
 
 sub test_intermediate_rename
     :AllowMoves :Replication :SyncLog :DelayedDelete :min_version_3_3
+    :needs_component_replication
 {
     my ($self) = @_;
 
@@ -1962,6 +1978,7 @@ sub test_intermediate_rename
 
 sub test_intermediate_upgrade
     :AllowMoves :Replication :SyncLog :DelayedDelete :min_version_3_3
+    :needs_component_replication
 {
     my ($self) = @_;
 
@@ -1986,6 +2003,7 @@ sub test_intermediate_upgrade
 
 sub test_clean_remote_shutdown_while_rolling
     :CSyncReplication :SyncLog :min_version_3_5
+    :needs_component_replication
 {
     my ($self) = @_;
 
@@ -2040,6 +2058,7 @@ sub test_clean_remote_shutdown_while_rolling
 
 sub test_rolling_retry_wait_limit
     :CSyncReplication :NoStartInstances :min_version_3_5
+    :needs_component_replication
 {
     my ($self) = @_;
     my $maxwait = 20;
@@ -2097,7 +2116,7 @@ sub test_rolling_retry_wait_limit
 # Test empty mailbox gets overwritten
 #
 sub test_splitbrain_different_uniqueid_unused
-    :min_version_3_5
+    :min_version_3_5 :needs_component_replication
 {
     my ($self) = @_;
 
@@ -2136,7 +2155,7 @@ sub test_splitbrain_different_uniqueid_unused
 # Test non-empty mailbox causes replication to abort
 #
 sub test_splitbrain_different_uniqueid_nonempty
-    :min_version_3_5
+    :min_version_3_5 :needs_component_replication
 {
     my ($self) = @_;
 
@@ -2190,7 +2209,7 @@ sub test_splitbrain_different_uniqueid_nonempty
 # Test mailbox that's had email but is now empty again
 #
 sub test_splitbrain_different_uniqueid_used
-    :min_version_3_5
+    :min_version_3_5 :needs_component_replication
 {
     my ($self) = @_;
 
@@ -2254,4 +2273,231 @@ sub test_splitbrain_different_uniqueid_used
     $self->assert_str_equals($mid, $rid);
 }
 
+sub test_delete_longname
+    :AllowMoves :Replication :SyncLog :DelayedDelete :min_version_3_3
+    :needs_component_replication
+{
+    my ($self) = @_;
+
+    my $mtalk = $self->{master_store}->get_client();
+
+    #define MAX_MAILBOX_NAME 490
+    my $name = "INBOX.this is a really long name 1 1 1 1 1 1 1 1 1 1 1 1 1 1 1 1 1 1 1 1 1 1 1 1 1 1 1 1 1 1 1 1 1 1 1 1 1 1 1 1 1 1 1 1 1 1 1 1 1 1 1 1 1 1 1 1 1 1 1 1 1 1 1 1 1 1 1 1 1 1 1 1 1 1 1 1 1 1 1 1 1 1 1 1 1 1 1 1 1 1 1 1 1 1 1 1 1 1 1 1 1 1 1 1 1 1 1 1 1.2 2 2 2 2 2 2 2 2 2 2 2 2 2 2 2 2 2 2 2 2 2 2 2 2 2 2 2 2 2 2 2 2 2 2 2 2 2 2 2 2 2 2 2 2 2 2 2 2.3 3 3 3 3 3 3 3 3 3 3 3 3 3 3 3 3 3 3 3 3 3 3 3 3 3 3 3 3 3 3 3 3 3 3 3 3 3 3 3 3 3 3 3 3 3 3 3 3 3 3 3 3 3 3 3 3 3 3 3 3.foo";
+    my ($success) = $mtalk->create($name);
+    die "Failed to create" unless $success;
+
+    # replicate and check initial state
+    my $synclogfname = "$self->{instance}->{basedir}/conf/sync/log";
+    $self->run_replication(rolling => 1, inputfile => $synclogfname);
+    unlink($synclogfname);
+
+    # reconnect
+    $mtalk = $self->{master_store}->get_client();
+
+    $mtalk->delete($name);
+
+    $self->run_replication(rolling => 1, inputfile => $synclogfname) if -f $synclogfname;
+
+    $self->check_replication('cassandane');
+}
+
+sub test_toarchive
+    :NoStartInstances :ArchivePartition :min_version_3_7
+    :needs_component_replication
+{
+    my ($self) = @_;
+
+    my $repcfg = $self->{replica}->{config};
+    $repcfg->set('debug_log_sync_partition_choice' => 'yes');
+    $self->_start_instances();
+
+    my $mtalk = $self->{master_store}->get_client();
+    $self->{master_store}->_select();
+    $self->assert_num_equals(1, $mtalk->uid());
+    $self->{master_store}->set_fetch_attributes(qw(uid flags));
+
+    xlog $self, "Append 3 old messages";
+    my %msg;
+    foreach my $id (1..3) {
+        my $olddate = DateTime->now();
+        $olddate->add(DateTime::Duration->new(months => -4 + $id));
+
+        $msg{$id} = $self->make_message("Message $id",
+                                        date => $olddate,
+                                        store => $self->{master_store});
+        $msg{$id}->set_attributes(id => $id,
+                                  uid => $id,
+                                  flags => []);
+    }
+
+    xlog $self, "Append 3 current messages";
+    foreach my $id (4..6) {
+        $msg{$id} = $self->make_message("Message $id",
+                                        store => $self->{master_store});
+        $msg{$id}->set_attributes(id => $id,
+                                  uid => $id,
+                                  flags => []);
+    }
+
+    xlog $self, "Run cyr_expire to archive old messages";
+    $self->{instance}->run_command({ cyrus => 1 }, 'cyr_expire', '-A' => '7d' );
+
+    $self->check_messages(\%msg);
+
+    my $mbpath = $self->{instance}->run_mbpath('-u', 'cassandane');
+    my $mdatadir = $mbpath->{data};
+    my $marchivedir = $mbpath->{archive};
+
+    foreach my $id (1..6) {
+        if ($id > 3) {
+            $self->assert(-f "$mdatadir/$id.");
+            $self->assert(! -f "$marchivedir/$id.");
+        }
+        else {
+            $self->assert(! -f "$mdatadir/$id.");
+            $self->assert(-f "$marchivedir/$id.");
+        }
+    }
+
+    $self->{replica}->getsyslog(); # discard setup noise
+
+    xlog $self, "Run replication, staging on archive partition";
+    $self->run_replication('stagetoarchive' => 1);
+    $self->check_replication('cassandane');
+
+    # ensure we made the correct choices about how to stage
+    if ($self->{replica}->{have_syslog_replacement}) {
+        my $part = $repcfg->substitute(
+            $repcfg->get('archivepartition-default')
+        );
+
+        my @choices = grep {
+            m{debug_log_sync_partition_choice: chose reserve path}
+        } $self->{replica}->getsyslog();
+
+        $self->assert_num_equals(scalar(keys %msg), scalar @choices);
+
+        foreach my $choice (@choices) {
+            $self->assert_matches(qr{\bbase=<$part>},
+                                  $choice);
+            $self->assert_matches(qr{\breserve_path=<$part/sync\./},
+                                  $choice);
+        }
+    }
+
+    # ensure that data ends up in the correct places
+    $mbpath = $self->{replica}->run_mbpath('-u', 'cassandane');
+    my $rdatadir = $mbpath->{data};
+    my $rarchivedir = $mbpath->{archive};
+
+    foreach my $id (1..6) {
+        if ($id > 3) {
+            $self->assert(-f "$rdatadir/$id.");
+            $self->assert(! -f "$rarchivedir/$id.");
+        }
+        else {
+            $self->assert(! -f "$rdatadir/$id.");
+            $self->assert(-f "$rarchivedir/$id.");
+        }
+    }
+}
+
+sub test_toarchive_noarchive
+    :NoStartInstances :min_version_3_7 :needs_component_replication
+{
+    my ($self) = @_;
+
+    my $repcfg = $self->{replica}->{config};
+    $repcfg->set('debug_log_sync_partition_choice' => 'yes');
+    $self->_start_instances();
+
+    my $mtalk = $self->{master_store}->get_client();
+    $self->{master_store}->_select();
+    $self->assert_num_equals(1, $mtalk->uid());
+    $self->{master_store}->set_fetch_attributes(qw(uid flags));
+
+    xlog $self, "Append 3 old messages";
+    my %msg;
+    foreach my $id (1..3) {
+        my $olddate = DateTime->now();
+        $olddate->add(DateTime::Duration->new(months => -4 + $id));
+
+        $msg{$id} = $self->make_message("Message $id",
+                                        date => $olddate,
+                                        store => $self->{master_store});
+        $msg{$id}->set_attributes(id => $id,
+                                  uid => $id,
+                                  flags => []);
+    }
+
+    xlog $self, "Append 3 current messages";
+    foreach my $id (4..6) {
+        $msg{$id} = $self->make_message("Message $id",
+                                        store => $self->{master_store});
+        $msg{$id}->set_attributes(id => $id,
+                                  uid => $id,
+                                  flags => []);
+    }
+
+    xlog $self, "Run cyr_expire to archive old messages";
+    $self->{instance}->run_command({ cyrus => 1 }, 'cyr_expire', '-A' => '7d' );
+
+    $self->check_messages(\%msg);
+
+    my $mbpath = $self->{instance}->run_mbpath('-u', 'cassandane');
+    my $mdatadir = $mbpath->{data};
+    my $marchivedir = $mbpath->{archive};
+
+    # expect data and archive should be the same
+    $self->assert_str_equals($mdatadir, $marchivedir);
+
+    # all messages in same place
+    foreach my $id (1..6) {
+        $self->assert(-f "$mdatadir/$id.");
+    }
+
+    $self->{replica}->getsyslog(); # discard setup noise
+
+    xlog $self, "Run replication, staging on archive partition";
+    $self->run_replication('stagetoarchive' => 1);
+    $self->check_replication('cassandane');
+
+    # ensure we made the correct choices about how to stage
+    if ($self->{replica}->{have_syslog_replacement}) {
+        my $part = $repcfg->substitute(
+            $repcfg->get('partition-default')
+        );
+
+        my @choices = grep {
+            m{debug_log_sync_partition_choice: chose reserve path}
+        } $self->{replica}->getsyslog();
+
+        $self->assert_num_equals(scalar(keys %msg), scalar @choices);
+
+        foreach my $choice (@choices) {
+            $self->assert_matches(qr{\bbase=<$part>},
+                                  $choice);
+            $self->assert_matches(qr{\breserve_path=<$part/sync\./},
+                                  $choice);
+        }
+    }
+
+    # ensure that data ends up in the correct places
+    $mbpath = $self->{replica}->run_mbpath('-u', 'cassandane');
+    my $rdatadir = $mbpath->{data};
+    my $rarchivedir = $mbpath->{archive};
+
+    # expect data and archive should be the same
+    $self->assert_str_equals($rdatadir, $rarchivedir);
+
+    # all messages in same place
+    foreach my $id (1..6) {
+        $self->assert(-f "$rdatadir/$id.");
+    }
+
+    foreach my $id (1..6) {
+        $self->assert(-f "$rdatadir/$id.");
+    }
+}
+
 1;
