@@ -138,6 +138,47 @@ sub id
     return $self->{_name}; # XXX something cleverer?
 }
 
+sub filter
+{
+    my ($self) = @_;
+
+    my $filter = $self->SUPER::filter(@_);
+
+    $filter->{enable_wanted_properties} = sub {
+        return if not exists $self->{_name};
+        my $sub = $self->can($self->{_name});
+        return if not defined $sub;
+
+        # n.b. cannot be used to unwant, sorry
+        foreach my $attr (attributes::get($sub)) {
+            next if $attr !~ m/^want(_service)?_(\w+)$/;
+
+            # XXX Since this is a 'filter', it could also check
+            # XXX whether the required components are configured
+            # XXX ala :needs_foo, and skip the test if they're
+            # XXX missing, rather than failing to start them.
+            # XXX That is, :want_service_http could be taken to
+            # XXX imply :needs_component_httpd, and the test
+            # XXX skipped if it's unavailable.  But note that
+            # XXX there isn't a clean mapping between the names!
+            # XXX For now, tests will need to be annotated with
+            # XXX both attributes in these cases.
+
+            $self->{_current_magic} = "Test function attribute ':$attr'";
+            if (defined $1 && $1 eq '_service') {
+                $self->want_services($2);
+            }
+            else {
+                $self->want($2);
+            }
+            $self->{_current_magic} = undef;
+        }
+        return;
+    };
+
+    return $filter;
+}
+
 # will magically cause some special actions to be taken during test
 # setup.  This used to be a horrible hack to enable a replica instance
 # if the test name contained the word "replication", but now it's more
@@ -190,6 +231,7 @@ sub config_set
     }
 }
 
+# XXX put these in alphabetical order someday
 magic(ReverseACLs => sub {
     shift->config_set(reverseacls => 1);
 });
@@ -378,6 +420,31 @@ magic(HttpJWTAuthRSA => sub {
     $self->config_set(http_jwt_key_dir => '@basedir@/conf/certs/http_jwt');
     $self->want('install_certificates');
 });
+magic(iCalendarMaxSize10k => sub {
+    shift->config_set(icalendar_max_size => 100000);
+});
+magic(CalDAVNoDefaultCalendar => sub {
+    shift->config_set(
+        caldav_create_default => 'no',
+    );
+});
+magic(AltPTSDBPath => sub {
+    shift->config_set(
+        'ptscache_db_path' => '@basedir@/conf/non-default-ptscache.db'
+    );
+});
+magic(ArchivePartition => sub {
+    my $conf = shift;
+    $conf->config_set('archivepartition-default' => '@basedir@/archive');
+    $conf->config_set('archive_enabled' => 'yes');
+    $conf->config_set('archive_after' => '7d');
+});
+magic(ArchiveNow => sub {
+    my $conf = shift;
+    $conf->config_set('archivepartition-default' => '@basedir@/archive');
+    $conf->config_set('archive_enabled' => 'yes');
+    $conf->config_set('archive_after' => '0d');
+});
 
 # Run any magic handlers indicated by the test name or attributes
 sub _run_magic
@@ -405,6 +472,8 @@ sub _run_magic
             next if $a =~ m/^(?:min|max)_version_/;
             # ignore feature test attribution here
             next if $a =~ m/^needs_/;
+            # ignore want attribution here
+            next if $a =~ m/^want_/;
             die "Unknown attribute $a"
                 unless defined $magic_handlers{$m};
             next if $seen{$m};
@@ -702,6 +771,10 @@ sub _setup_http_service_objects
     my ($self) = @_;
 
     # nothing to do if no http service
+    require Mail::JMAPTalk;
+    require Net::CalDAVTalk;
+    require Net::CardDAVTalk;
+
     my $service = $self->{instance}->get_service("http");
     return if !$service;
 
@@ -1162,6 +1235,7 @@ sub run_replication
     my $meta = delete $opts{meta};
     my $nosyncback = delete $opts{nosyncback};
     my $allusers = delete $opts{allusers};
+    my $stagetoarchive = delete $opts{stagetoarchive};
     $nmodes++ if $user;
     $nmodes++ if $rolling;
     $nmodes++ if $mailbox;
@@ -1196,6 +1270,7 @@ sub run_replication
     push(@cmd, '-u', $user) if defined $user;
     push(@cmd, '-m', $mailbox) if defined $mailbox;
     push(@cmd, '-A') if $allusers;  # n.b. boolean
+    push(@cmd, '-a') if $stagetoarchive; # n.b. boolean
 
     my %run_options;
     $run_options{cyrus} = 1;
