@@ -85,6 +85,19 @@ extern time_t icaltime_to_timet(icaltimetype t, const icaltimezone *floatingtz);
 extern void icalproperty_set_xparam(icalproperty *prop,
                                     const char *name, const char *val, int purge);
 
+/* Returns if default alerts are explicitly enabled (1) or disabled (0).
+   Returns -1 otherwise. */
+extern int icalcomponent_read_usedefaultalerts(icalcomponent *comp);
+
+extern void icalcomponent_set_usedefaultalerts(icalcomponent *comp);
+
+/* Adds default alerts to ical, if either the X-USE-DEFAULTALERTS property
+ * is set to TRUE, or force is non-zero. */
+extern void icalcomponent_add_defaultalerts(icalcomponent *ical,
+                                            icalcomponent *alarms_withtime,
+                                            icalcomponent *alarms_withdate,
+                                            int force);
+
 /* If range is a NULL period, callback() is executed for ALL occurrences,
    otherwise callback() is only executed for occurrences that overlap the range.
    callback() returns true (1) while it wants more occurrences, 0 to finish.
@@ -97,6 +110,8 @@ extern int icalcomponent_myforeach(icalcomponent *comp,
                                    int (*callback) (icalcomponent *comp,
                                                     icaltimetype start,
                                                     icaltimetype end,
+                                                    icaltimetype recurid,
+                                                    int is_standalone,
                                                     void *data),
                                    void *callback_data);
 
@@ -151,6 +166,32 @@ extern int icalcomponent_apply_vpatch(icalcomponent *ical,
 extern const char *icaltimezone_get_location_tzid(const icaltimezone *zone);
 extern const char *icaltime_get_location_tzid(icaltimetype t);
 
+extern icaltimezone *icaltimezone_get_cyrus_timezone_from_tzid(const char *tzid);
+
+struct observance {
+    const char *name;
+    icaltimetype onset;
+    int offset_from;
+    int offset_to;
+    int is_daylight;
+    int is_std;
+    int is_gmt;
+};
+
+extern void icaltimezone_truncate_vtimezone_advanced(icalcomponent *vtz,
+                                                     icaltimetype *startp, icaltimetype *endp,
+                                                     icalarray *obsarray,
+                                                     struct observance *proleptic,
+                                                     icalcomponent **eternal_std,
+                                                     icalcomponent **eternal_dst,
+                                                     icaltimetype *last_dtstart,
+                                                     int ms_compatible);
+
+extern int ical_categories_is_color(icalproperty *cprop);
+
+/* Normalizes both standard and cyrus-extensions */
+extern void icalcomponent_normalize_x(icalcomponent *ical);
+
 /* Functions that should be declared in libical */
 #define icaltimezone_set_zone_directory set_zone_directory
 
